
#include <config.h>
#include <gnome.h>
#include <malloc.h>

GtkWidget *make_menus();
void show_dialog();
void about_cb();
void apply_cb();
void activate_cb();
void create_property_box();

int g_argc;
char **g_argv;

char *app_title = NULL;
gboolean show_mouse_droppings = 0;
gboolean show_grid = 0;
char message[100];

gint save_yourself_cb(GnomeClient *client, gint phase, 
    GnomeRestartStyle save_style, gint shutdown, 
    GnomeInteractStyle interact_style, gint fast)
{
    /* Command to remove old config data for this session */
    gchar *discard_argv[] = { "rm", "-rf", NULL };
    gchar *prefix;

    g_message("Running save_yourself handler");

    g_assert(GNOME_CLIENT_CONNECTED(client));
    prefix = gnome_client_get_config_prefix(client);
    discard_argv[2] = gnome_config_get_real_path(prefix);

    g_message("Registering session commands...");
    gnome_client_set_discard_command(client, 3, discard_argv);
    gnome_client_set_restart_command(client, g_argc, g_argv);
    gnome_client_set_clone_command(client, g_argc, g_argv);

    g_message("Saving config state from session...");
    save_config_state(prefix);

    return TRUE;
}

void die_cb(GnomeClient *client)
{
  g_message("Running die handler");
  gtk_main_quit();
};

void save_config_state(gchar *prefix)
{
    g_message("Saving config state...");
    gnome_config_push_prefix(prefix);

    sprintf(message,"title is %s\n",app_title);
    g_message(message);
    gnome_config_set_string("/GnomeStroke/Settings/Title", app_title);
    sprintf(message,"title is %s\n",app_title);
    g_message(message);
    gnome_config_set_bool("/GnomeStroke/Settings/ShowMouseDroppings", show_mouse_droppings);
    gnome_config_set_bool("/GnomeStroke/Settings/ShowGrid", show_grid);

    gnome_config_pop_prefix();
    gnome_config_sync();
}

void load_config_state(gchar *prefix)
{
    gboolean got_default;

    gnome_config_push_prefix(prefix);

    if (show_mouse_droppings == 0)
      show_mouse_droppings = 
        gnome_config_get_bool("ShowMouseDroppings=true");
    if (show_grid == 0)
      show_grid = 
        gnome_config_get_bool("ShowGrid=true");
    if (app_title == NULL) {
      g_message("Loading title state\n");
      gnome_config_get_string("Title=");
    }
    sprintf(message,"title is %s\n",app_title);
    g_message(message);
  /*
    if (app_title == NULL) {
      g_message ("Setting default title");
      app_title = (char *) g_malloc(sizeof("Gnome Stroke Configurator"));
      strcpy (app_title,"Gnome Stroke Configurator");
    }
  */
    gnome_config_pop_prefix();
}

int main(int argc, char *argv[])
{
  GtkWidget *app;
  GtkWidget *app_bar = NULL;
  GtkWidget *button;
  gint result;
  GnomeClient *client;

  static const struct poptOption myapp_options[] =
  {
    { "title", 't', POPT_ARG_STRING, &app_title, 0, 
      N_("Application title"), N_("TITLE") },
    { "show_mouse_droppings", 'd', POPT_ARG_NONE, &show_mouse_droppings, 0, 
      N_("Show mouse droppings"), N_("") },
    { "show_grid", 'g', POPT_ARG_NONE, &show_grid, 0, 
      N_("Show grid"), N_("") },
    { NULL, '\0', 0, NULL, 0 }
  };

  gnome_init_with_popt_table("gnome_test", VERSION, argc, argv, myapp_options, 0, NULL);

  g_argc = argc;
  g_argv = argv;

  g_message("Registering for session management");
  client = gnome_master_client();
  gtk_signal_connect(GTK_OBJECT(client), "save_yourself", 
      GTK_SIGNAL_FUNC(save_yourself_cb), NULL);
  gtk_signal_connect(GTK_OBJECT(client), "save_yourself", 
      GTK_SIGNAL_FUNC(die_cb), NULL);

  if (GNOME_CLIENT_CONNECTED(client))
  {
      g_message("Loading config state for session...");
      load_config_state(gnome_client_get_config_prefix(client));
  }
  else
  { 
    if (show_mouse_droppings == 0)
      show_mouse_droppings = 
        gnome_config_get_bool("/GnomeStroke/Settings/ShowMouseDroppings=true");
    if (show_grid == 0)
      show_grid = 
        gnome_config_get_bool("/GnomeStroke/Settings/ShowGrid=true");
    if (app_title == NULL) {
      g_message("Loading title state\n");
      gnome_config_get_string("/GnomeStroke/Settings/Title=");
    }
    sprintf(message,"title is %s\n",app_title);
    g_message(message);
  /*
    if (app_title == NULL) {
      g_message ("Setting default title");
      app_title = (char *) g_malloc(sizeof("Gnome Stroke Configurator"));
      strcpy (app_title,"Gnome Stroke Configurator");
    }
  */
  }

  g_message("Creating app...\n");
  app = gnome_app_new("myapp", app_title);
  button = gtk_button_new_with_label("Quit Button");

  g_message("Connecting signals...\n");
  gtk_signal_connect(GTK_OBJECT(app), "delete_event", 
    GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
  gtk_signal_connect(GTK_OBJECT(button), "clicked", 
    GTK_SIGNAL_FUNC(gtk_main_quit), NULL); 
  sprintf(message,"droppings is %d, trail is %d\n",show_mouse_droppings,show_grid);
  g_message(message);

  gnome_app_set_contents(GNOME_APP(app), button);

  /* make the menus and an appbar */
  app_bar = make_menus(app);

  gtk_widget_show_all(app);

  g_message("Entering main loop...\n");
  gtk_main();

 /* We'll only reach this point if there's no session management, 
     so must save things manually (save_yourself_cb won't be called) */
  if (!GNOME_CLIENT_CONNECTED(client))
  {
    sprintf(message,"title is %s\n",app_title);
    g_message(message);
    gnome_config_set_string("/GnomeStroke/Settings/Title", app_title);
    sprintf(message,"title is %s\n",app_title);
    g_message(message);
    gnome_config_set_bool("/GnomeStroke/Settings/ShowMouseDroppings", show_mouse_droppings);
    gnome_config_set_bool("/GnomeStroke/Settings/ShowGrid", show_grid);
    gnome_config_sync();
  }

  return 0;
}

void show_dialog() {
  GtkWidget *dialog;
  GtkWidget *label;
  GtkWidget *entry;
  gint result;

  dialog = gnome_dialog_new("Name Dialog", GNOME_STOCK_BUTTON_OK,
    GNOME_STOCK_BUTTON_CANCEL, NULL);
  label = gtk_label_new("Please enter your name");
  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
    label, TRUE, TRUE, 0);
  entry = gnome_entry_new("name_history_id");
  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox),
    entry, TRUE, TRUE, 0);

  gnome_dialog_set_close (GNOME_DIALOG(dialog),TRUE);
  result = gnome_dialog_run(GNOME_DIALOG(dialog));
/*  gnome_dialog_close(dialog); */

  if(result == 0)
    g_message("Hit the OK Button");
  else if (result == 1)
    g_message("Hit the Cancel Button");
  else  /* result == -1 */
    g_message("Hit Escape key or closed with window manager");

  return;
}

GtkWidget *make_menus(GnomeApp *app) {

  GtkWidget *app_bar = NULL;

  GnomeUIInfo file_menu[] = 
  {
    GNOMEUIINFO_MENU_NEW_ITEM(N_("_New Window"), 
      N_("Create a New Configurator Window"), show_dialog, NULL),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_MENU_EXIT_ITEM(gtk_main_quit, NULL),
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_MENU_PROPERTIES_ITEM(create_property_box, NULL),
    GNOMEUIINFO_END
  };  

  GnomeUIInfo help_menu[] = 
  {
    GNOMEUIINFO_HELP("GnomeStroke"),
    GNOMEUIINFO_MENU_ABOUT_ITEM(about_cb, NULL),
    GNOMEUIINFO_END
  };

  GnomeUIInfo main_menu[] = 
  {
    GNOMEUIINFO_MENU_FILE_TREE(file_menu),
    GNOMEUIINFO_MENU_HELP_TREE(help_menu),
    GNOMEUIINFO_END
  };


  gnome_app_create_menus(app,(GnomeUIInfo *)main_menu);


  /* make an appbar */
  app_bar = gnome_appbar_new(FALSE, TRUE, GNOME_PREFERENCES_USER);
  gnome_app_set_statusbar(app,app_bar);

  return app_bar;
}

void about_cb(GtkWidget *menuitem, gpointer data)
{
  const gchar *authors[] = 
  { "Mark Willey", NULL };

  GtkWidget *about;
  about = gnome_about_new(_("About GNOME Stroke"), VERSION, 
    "(C) 1997-1999 ETLA Technical Services.", authors,
    _("GNOME Stroke is a utility that translates pointer "
    "motions into system commands.\n"
    "http://www.etla.net/~willey/projects/libstroke/"),
    NULL);

  gtk_widget_show(GTK_WIDGET(about));

  return;
}

void apply_cb(GnomePropertyBox *propertybox, gint page_num)
{
  switch(page_num)
  {
  case 0:
    g_message("Applying first page");
    break;
  case 1:
    g_message("Applying second page");
    break;
  case -1:
    g_message("Applying all pages");
    break;
  }
}

void activate_cb(GtkEditable *editable, gpointer data)
{
  GnomePropertyBox *pbox = GNOME_PROPERTY_BOX(data);
  gnome_property_box_changed(pbox);
  g_message("Activated...");
}

void create_property_box()
{
  GnomePropertyBox *pbox;
  GtkWidget *page;
  GtkWidget *label;
  GtkWidget *gtkentry;

  pbox = GNOME_PROPERTY_BOX(gnome_property_box_new());

  /* Set up first page: a single GnomeEntry
  /* Also, connect to GtkEntry's "activate" signal...
   * must hit Enter in entry widget to trigger it
   */
  page = gnome_entry_new("Grid Size");
  gtkentry = gnome_entry_gtk_entry(GNOME_ENTRY(page));
  gtk_signal_connect(GTK_OBJECT(gtkentry), "activate",
    GTK_SIGNAL_FUNC(activate_cb), pbox);
  label = gtk_label_new("Size of Grid");
  gnome_property_box_append_page(pbox, page, label);

  /* Set up second page */
  page = gnome_entry_new("Droppings Color");
  gtkentry = gnome_entry_gtk_entry(GNOME_ENTRY(page));
  gtk_signal_connect(GTK_OBJECT(gtkentry), "activate",
    GTK_SIGNAL_FUNC(activate_cb), pbox);
  label = gtk_label_new("Mouse Droppings Color");
  gnome_property_box_append_page(pbox, page, label);

  /* Connect "apply" signal to commit changes */
  gtk_signal_connect(GTK_OBJECT(pbox), "apply",
    GTK_SIGNAL_FUNC(apply_cb), NULL);

  gtk_widget_show_all(GTK_WIDGET(pbox));
}

